/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.TYPE        Module
.NAME        tblconv.c
.LANGUAGE    C
.AUTHOR      J.D.Ponz  ESA/VILSPA
.CATEGORY    IUEFA
.PURPOSE     Converts a 3D table file with MELORES spectra into
             one or several 2D spectral tables.
.COMMENTS    Command syntax:
                   tblconv intab 
             where 'intab' is the input table. The program creates a table 
             per aperture. The name of the output table is generated by
             adding a letter to the input name to indicate the aperture.
.VERSION     1.0 25-Mar-1993 JDP  Implementation     
.VERSION     1.1 05-Apr-1993 IY   Get out of MIDAS environment
------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <tbldef.h>
#include <midas_def.h>


#define PARLEN 80

/*=======================================================================*/

main(argc,argv)
 int argc;     /* IN: No. of arguments in input command string */
 char *argv[]; /* IN: Pointer to contents of args. in inp. cmd. string */
{
  int   actval, tid, tid1, ncol, nrow, nsort, allcol, allrow;
  int   npoints, i, j, jp4, k, ncols, ic, len, dtype, dnull, ival, k1;
  char   *dunit;
  float  vals[1000];                       /* float values for spectra */
  int   ivals[1000];                      /* associated flags */
  float  wave, delta, rval;
  double w;                 
  char   table_name[PARLEN];
  char   apert[10];
  char   label[17];
  char   unit[17];
  char   form[17];
  char  *new_name(), *table_out;

 /* Standalone Midas env. No Midas session needed */
  SCSPRO("-1");

 /* read input table name */
  if(argc != 2) {
     fprintf(stderr," Usage: tblconv tablename\n");
     exit(0);
     }
  strcpy(table_name,argv[1]); 

  if (TCTOPN(table_name, F_IO_MODE, &tid))
      {SCTPUT ("**** Error: TCTOPN"); ospexit(0);}
                                          /* read table information */
  if (TCIGET(tid, &ncol, &nrow, &nsort, &allcol, &allrow))
      {SCTPUT ("**** Error: TCIGET"); ospexit(0);}
                                          /* check table size */
  if (ncol < 1 || ncol > 9)
     {SCTPUT ("**** Error: wrong number of columns"); ospexit(0);}
  if (nrow < 1 || nrow > 2)
      {SCTPUT ("**** Error: wrong number of rows"); ospexit(0);}
                                          /* loop to process each spectrum */
  for (i = 1; i <= nrow; i++)
   {TCERDC(tid, i, 1L, apert, &dnull);
    if (strncmp(apert, "LARGE", 5)==0) table_out = new_name(table_name, "L"); 
    else if (strncmp(apert,"SMALL",5)==0) table_out = new_name(table_name, "S");
    else             {SCTPUT ("**** Error: wrong input table "); ospexit(0);}
                                              /* read sampling domain */
    TCERDI(tid, i, 2L, &npoints, &dnull);
    if (npoints != 640) {SCTPUT ("**** Error: bad number of bins"); ospexit(0);}
    TCERDR(tid, i, 3L, &wave, &dnull);
    TCERDR(tid, i, 4L, &delta, &dnull);
                                              /* create output table */
    ncols = 6;
    allcol = 9;
    if (TCTINI(table_out, F_TRANS, F_O_MODE, allcol, npoints, &tid1))
         {SCTPUT ("**** Error TCTINI"); ospexit(0);}
                                              /* generate wavelength column */
    TCCINI(tid1, D_R8_FORMAT, 1L, "F10.3", "ANGSTROM", "WAVELENGTH", &ic);
    for (j = 1; j <= npoints; j++)
       {w = wave + (j-1)*delta; TCEWRD(tid1, j, ic, &w);}

    for (j = 1; j < ncols; j++)              /* generate rest of columns */
        {jp4 = j+4; 
         TCFGET(tid, jp4, form, &len, &dtype);
         TCLGET(tid, jp4, label);
         TCUGET(tid, jp4, unit);
         TCCINI(tid1, dtype, 1L, form, unit, label, &ic);
         switch (dtype) {
         case D_R4_FORMAT:                          /* float values */
             TCARDR(tid, i, jp4, 1L, npoints, vals);
             for (k=0; k < npoints; k++) 
                    {rval = vals[k]; k1 = k+1; TCEWRR(tid1, k1, ic, &rval);}
             break;
         case D_I2_FORMAT:                          /* integer values */
             TCARDI(tid, i, jp4, 1L, npoints, ivals);
             for (k=0; k < npoints; k++) 
                    {ival = ivals[k]; k1 = k+1; TCEWRI(tid1, k1, ic, &ival);}
             break;
         default:                                   /* never here */
             SCTPUT ("**** Error: wrong column type");
             ospexit(0);
         } 
       }
    if (TCTCLO(tid1)) {SCTPUT ("**** Error TCTCLO"); ospexit(0);}
  } 
                      
  if (TCTCLO(tid))                           /* close input table */
  {SCTPUT ("**** Error TCTCLO"); ospexit(0);}

  ospexit(0);
}

char *new_name(name, ap)        /* generate filename with aperture code */
char   *name;
char   ap[];
{
 char  *pc, *pcx;
 static char xname[PARLEN];

   pc = xname; pcx = (char *) 0;
   while (*pc = *name++) 
      if (*pc++ == '.') {pc--; *pc++ = ap[0]; *pc++ = '.'; pcx = pc;} 
   if (!pcx) *pc++ = ap[0];            /* if no '.' add aperture code at end */
   *pc = '\0';
   return xname;
}
